<?php

/**
 * Class ControllerExtensionPaymentVtbPay
 * This class controls the VTB payment extension.
 * It includes methods for rendering the payment settings page,
 * processing the form submissions, validating the inputs, etc.
 */
class ControllerExtensionPaymentVtbPay extends Controller
{
    private $error = [];


    /**
     * Install method
     *
     * Called on an install event
     *
     * @return void
     */
    public function install() {
        $this->load->model('extension/payment/vtbpay');
        $this->model_extension_payment_vtbpay->install();
    }


    /**
     * Uninstall method
     *
     * Called on a delete event
     *
     * @return void
     */
    public function uninstall() {
        $this->load->model('extension/payment/vtbpay');
        $this->model_extension_payment_vtbpay->uninstall();
    }

    /**
     * getFormValue method
     *
     * This method fetches the form value from the request or the configuration.
     * It checks if a particular key is set in the request,
     * if it's set then it returns the corresponding value,
     * otherwise, it fetches the value from the config.
     *
     * @param array $request Array containing form data.
     * @param object $config OpenCart Config object.
     * @param string $key Key to look for in the request and config.
     * @return string The value of the provided key.
     */
    private function getFormValue($request, $config, $key)
    {
        return trim($request[$key] ?? $config->get($key));
    }


    /**
     * getCheckboxValue method
     *
     * This method fetches the checkbox value from the request or the configuration.
     * It fetches the form value and checks if it's truthy,
     * then it returns '1', otherwise, it returns '0'.
     *
     * @param array $request Array containing form data.
     * @param object $config OpenCart Config object.
     * @param string $key Key to look for in the request and config.
     * @return string The value ('1' or '0') of the provided key.
     */
    private function getCheckboxValue($request, $config, $key)
    {
        return $this->getFormValue($request, $config, $key) ? '1' : '0';
    }


    /**
     * index method
     *
     * This is the default method for the controller.
     * It loads the language file, the setting model, the order status model,
     * handles the form submission, and prepares the data to render the settings page.
     *
     * @return void
     */
    public function index()
    {
        $this->load->language('extension/payment/vtbpay');
        $this->document->setTitle($this->language->get('heading_title'));
        $this->load->model('setting/setting');

        // If the request is POST and validation is passed, then save the changes
        if (
            ($this->request->server['REQUEST_METHOD'] == 'POST') &&
            $this->validate()
        ) {
            $this->model_setting_setting->editSetting('payment_vtbpay', $this->request->post);
            $this->session->data['success'] = $this->language->get('text_success');
            $this->response->redirect($this->url->link(
                'extension/payment/vtbpay',
                'user_token=' . $this->session->data['user_token'] . '&type=payment',
                true
            ));
        }

        // set value for form fields
        $fields = $this->getSettingsFields();

        foreach ($fields as $field => $field_data) {
            $data['settings']['payment_vtbpay_' . $field] = $field_data;
            if ($field_data['type'] === 'separator') continue;
            if ($field_data['type'] === 'checkbox') {
                $data['settings']['payment_vtbpay_' . $field]['value'] = $this->getCheckboxValue(
                    $this->request->post,
                    $this->config,
                    'payment_vtbpay_' . $field
                );
            }
            else {
                $data['settings']['payment_vtbpay_' . $field]['value'] = $this->getFormValue(
                    $this->request->post,
                    $this->config,
                    'payment_vtbpay_' . $field
                );
            }
        }

        $this->showPaymentSettingsPage($data);
    }


    /**
     * getPreparedOrderStatuses method
     *
     * Gets an array of order statuses prepared for display
     *
     * @return array
     */
    private function getPreparedOrderStatuses()
    {
        $this->load->model('localisation/order_status');

        $order_statuses = $this->model_localisation_order_status->getOrderStatuses();

        return array_combine(
            array_column($order_statuses, 'order_status_id'),
            array_column($order_statuses, 'name')
        );
    }


    /**
     * showPaymentSettingsPage method
     *
     * This method prepares the breadcrumbs and other necessary data for the view,
     * and then it renders the payment settings page.
     *
     * @param array $data The data array that will be passed to the view.
     * @return void
     */
    private function showPaymentSettingsPage($data)
    {
        $data['breadcrumbs'] = [];

        $data['breadcrumbs'][] = [
            'text' => $this->language->get('text_home'),
            'href' => $this->url->link('common/dashboard', 'user_token=' . $this->session->data['user_token'], true)
        ];

        $data['breadcrumbs'][] = [
            'text' => $this->language->get('text_extension'),
            'href' => $this->url->link('marketplace/extension', 'user_token=' . $this->session->data['user_token'] . '&type=payment', true)
        ];

        $data['breadcrumbs'][] = [
            'text' => $this->language->get('heading_title'),
            'href' => $this->url->link('extension/payment/vtbpay', 'user_token=' . $this->session->data['user_token'], true)
        ];

        $data['action'] = $this->url->link('extension/payment/vtbpay', 'user_token=' . $this->session->data['user_token'], true);
        $data['cancel'] = $this->url->link('marketplace/extension', 'user_token=' . $this->session->data['user_token'] . '&type=payment', true);

        $data['header'] = $this->load->controller('common/header');
        $data['column_left'] = $this->load->controller('common/column_left');
        $data['footer'] = $this->load->controller('common/footer');

        $data = $this->setAlert($data);

        // Load view template
        $this->response->setOutput($this->load->view('extension/payment/vtbpay', $data));
    }


    /**
     * getSettingsFields method
     *
     * Getting an array of plugin settings
     *
     * @return array
     */
    private function getSettingsFields()
    {
        return [
            'desc' => [
                'type' => 'textarea',
                'title' => $this->language->get('entry_desc'),
                'default' => $this->language->get('default_value_desc'),
                'desc' => $this->language->get('desc_desc')
            ],
            'client_id' => [
                'type' => 'text',
                'title' => $this->language->get('entry_client_id'),
                'required' => true,
                'desc' => $this->language->get('desc_client_id')
            ],
            'client_secret' => [
                'type' => 'text',
                'title' => $this->language->get('entry_client_secret'),
                'required' => true,
                'desc' => $this->language->get('desc_client_secret')
            ],
            'merchant_authorization' => [
                'type' => 'text',
                'title' => $this->language->get('entry_merchant_authorization'),
                'desc' => $this->language->get('desc_merchant_authorization')
            ],
            'successful_transaction_status' => [
                'type' => 'select',
                'title' => $this->language->get('entry_successful_transaction_status'),
                'desc' => $this->language->get('desc_successful_transaction_status'),
                'default' => '15',
                'options' => $this->getPreparedOrderStatuses()
            ],
            'failed_transaction_status' => [
                'type' => 'select',
                'title' => $this->language->get('entry_failed_transaction_status'),
                'desc' => $this->language->get('desc_failed_transaction_status'),
                'default' => '10',
                'options' => $this->getPreparedOrderStatuses()
            ],
            'test_mode' => [
                'type' => 'checkbox',
                'title' => $this->language->get('entry_test_mode'),
                'desc' => $this->language->get('desc_test_mode')
            ],
            'logging' => [
                'type' => 'checkbox',
                'title' => $this->language->get('entry_logging'),
                'desc' => $this->language->get('desc_logging')
            ],
            'two_stage' => [
                'type' => 'checkbox',
                'title' => $this->language->get('entry_two_stage'),
                'desc' => $this->language->get('desc_two_stage')
            ],
            'status' => [
                'type' => 'checkbox',
                'title' => $this->language->get('entry_status')
            ],
            'sort_order' => [
                'type' => 'text',
                'title' => $this->language->get('entry_sort_order')
            ],
            'section_fiscal' => [
                'type' => 'separator',
                'title' => $this->language->get('entry_section_fiscal')
            ],
            'enable_fiscal' => [
                'type' => 'checkbox',
                'title' => $this->language->get('entry_enable_fiscal'),
                'desc' => $this->language->get('desc_enable_fiscal')
            ],
            'email_fiscal' => [
                'type' => 'text',
                'title' => $this->language->get('entry_email_fiscal'),
                'desc' => $this->language->get('desc_email_fiscal')
            ],
            'payment_type_delivery_fiscal' => [
                'type' => 'select',
                'title' => $this->language->get('entry_payment_type_delivery_fiscal'),
                'desc' => $this->language->get('desc_payment_type_delivery_fiscal'),
                'default' => 'full_prepayment',
                'options' => [
                    'full_prepayment' => $this->language->get('option_full_prepayment'),
                    'prepayment' => $this->language->get('option_prepayment'),
                    'advance' => $this->language->get('option_advance'),
                    'full_payment' => $this->language->get('option_full_payment'),
                    'partial_payment' => $this->language->get('option_partial_payment'),
                    'credit' => $this->language->get('option_credit'),
                    'credit_payment' => $this->language->get('option_credit_payment')
                ]
            ],
            'section_subtitle_fiscal' => [
                'type' => 'separator',
                'title' => $this->language->get('entry_section_subtitle_fiscal')
            ],
            'measure_fiscal' => [
                'type' => 'select',
                'title' => $this->language->get('entry_measure_fiscal'),
                'desc' => $this->language->get('desc_measure_fiscal'),
                'default' => '0',
                'options' => [
                    '0' => $this->language->get('option_piece_unit'),
                    '10' => $this->language->get('option_gram'),
                    '11' => $this->language->get('option_kilogram'),
                    '12' => $this->language->get('option_ton'),
                    '20' => $this->language->get('option_centimeter'),
                    '21' => $this->language->get('option_decimeter'),
                    '22' => $this->language->get('option_meter'),
                    '30' => $this->language->get('option_sq_cm'),
                    '31' => $this->language->get('option_sq_dm'),
                    '32' => $this->language->get('option_sq_m'),
                    '40' => $this->language->get('option_ml'),
                    '41' => $this->language->get('option_liter'),
                    '42' => $this->language->get('option_cubic_m'),
                    '50' => $this->language->get('option_kwh'),
                    '51' => $this->language->get('option_gcal'),
                    '70' => $this->language->get('option_day'),
                    '71' => $this->language->get('option_hour'),
                    '72' => $this->language->get('option_minute'),
                    '73' => $this->language->get('option_second'),
                    '80' => $this->language->get('option_kb'),
                    '81' => $this->language->get('option_mb'),
                    '82' => $this->language->get('option_gb'),
                    '83' => $this->language->get('option_tb'),
                    '255' => $this->language->get('option_other_units')
                ]
            ],
            'tax_type_fiscal' => [
                'type' => 'select',
                'title' => $this->language->get('entry_tax_type_fiscal'),
                'desc' => $this->language->get('desc_tax_type_fiscal'),
                'default' => 'none',
                'options' => [
                    'none' => $this->language->get('option_no_vat'),
                    'vat0' => $this->language->get('option_vat0'),
                    'vat10' => $this->language->get('option_vat10'),
                    'vat110' => $this->language->get('option_vat110'),
                    'vat20' => $this->language->get('option_vat20'),
                    'vat120' => $this->language->get('option_vat120')
                ]
            ],
            'payment_type_fiscal' => [
                'type' => 'select',
                'title' => $this->language->get('entry_payment_type_fiscal'),
                'desc' => $this->language->get('desc_payment_type_fiscal'),
                'default' => 'full_prepayment',
                'options' => [
                    'full_prepayment' => $this->language->get('option_full_prepayment'),
                    'prepayment' => $this->language->get('option_prepayment'),
                    'advance' => $this->language->get('option_advance'),
                    'full_payment' => $this->language->get('option_full_payment'),
                    'partial_payment' => $this->language->get('option_partial_payment'),
                    'credit' => $this->language->get('option_credit'),
                    'credit_payment' => $this->language->get('option_credit_payment')
                ]
            ],
            'payment_subject_fiscal' => [
                'type' => 'select',
                'title' => $this->language->get('entry_payment_subject_fiscal'),
                'desc' => $this->language->get('desc_payment_subject_fiscal'),
                'default' => '1',
                'options' => [
                    '1' => $this->language->get('option_goods'),
                    '2' => $this->language->get('option_excise_goods'),
                    '3' => $this->language->get('option_work'),
                    '4' => $this->language->get('option_service'),
                    '5' => $this->language->get('option_gambling_bet'),
                    '6' => $this->language->get('option_gambling_win'),
                    '7' => $this->language->get('option_lottery_bet'),
                    '8' => $this->language->get('option_lottery_win'),
                    '9' => $this->language->get('option_rights'),
                    '10' => $this->language->get('option_payment'),
                    '11' => $this->language->get('option_agency_fee'),
                    '12' => $this->language->get('option_payout'),
                    '13' => $this->language->get('option_other_subject'),
                    '14' => $this->language->get('option_property_right'),
                    '15' => $this->language->get('option_non_operating_income'),
                    '16' => $this->language->get('option_insurance_contributions'),
                    '17' => $this->language->get('option_trade_fee'),
                    '18' => $this->language->get('option_resort_fee'),
                    '19' => $this->language->get('option_deposit'),
                    '20' => $this->language->get('option_expense'),
                    '21' => $this->language->get('option_pension_ip'),
                    '22' => $this->language->get('option_pension'),
                    '23' => $this->language->get('option_health_ip'),
                    '24' => $this->language->get('option_health'),
                    '25' => $this->language->get('option_social'),
                    '26' => $this->language->get('option_casino_payment'),
                    '27' => $this->language->get('option_cash_issuance_bank_agent'),
                    '30' => $this->language->get('option_excise_goods_no_code'),
                    '31' => $this->language->get('option_excise_goods_with_code'),
                    '32' => $this->language->get('option_goods_no_code'),
                    '33' => $this->language->get('option_goods_with_code')
                ]
            ]
        ];
    }


    /**
     * setAlert method
     *
     * Method that sets the display of alerts:
     * error messages and a message about successful saving
     *
     * @param array $data Array containing Output data.
     * @return array
     */
    protected function setAlert($data)
    {
        if (empty($this->error)) {
            // successful save output
            if (isset($this->session->data['success'])) {
                $data['success'] = $this->session->data['success'];
                unset($this->session->data['success']);
            }
        }
        else {
            $data['error'] = $this->error;
        }

        return $data;
    }


    /**
     * validate method
     *
     * This method validates the form submission.
     * It checks if the user has the permission to modify the settings,
     * validates the necessary fields
     *
     * @return bool Returns true if no validation error, otherwise returns false.
     */
    protected function validate()
    {
        if (!$this->user->hasPermission('modify', 'extension/payment/vtbpay')) {
            $this->error['warning'] = $this->language->get('error_msg_permission');
        }

        $required_fields = [
            'client_id',
            'client_secret'
        ];

        foreach ($required_fields as $required_field) {
            if (!$this->request->post['payment_vtbpay_' . $required_field]) {
                $this->error['payment_vtbpay_' . $required_field] = $this->language->get('error_msg_' . $required_field);
            }
        }

        // Return true if no error, false otherwise
        return empty($this->error);
    }
}
