<?php
use \Vtbpay\Classes\Exception\VtbPayException;

class ControllerExtensionPaymentVtbPay extends Controller
{
    private Vtbpay $vtbpay_lib;
    private array $order;


    /**
     * ControllerExtensionPaymentVtbPay constructor
     *
     * Set up the logging functionality for the VtbPay transactions.
     *
     * @param object $registry The registry of the OpenCart system
     */
    public function __construct($registry)
    {
        parent::__construct($registry);
        $this->load->language('extension/payment/vtbpay');

        $this->load->library('vtbpay');
        $this->vtbpay_lib = new Vtbpay();
        $this->vtbpay_lib->setVtbPayApi(
            $this->config->get('payment_vtbpay_client_id'),
            $this->config->get('payment_vtbpay_client_secret'),
            (bool) $this->config->get('payment_vtbpay_test_mode'),
            $this->config->get('payment_vtbpay_merchant_authorization') ?: ''
        )->setVtbPayLogger(
            (bool) $this->config->get('payment_vtbpay_logging')
        );
    }


    /**
     * Index function
     *
     * Prepares data for the checkout view, and returns the rendered HTML.
     *
     * @return string Rendered HTML of the checkout view
     */
    public function index()
    {
        $data['button_confirm'] = $this->language->get('button_confirm');
        $data['action'] = $this->url->link('extension/payment/vtbpay/confirm', '', true);
        $data['logo'] = '/catalog/view/theme/default/image/extension/payment/vtbpay.png';
        $data['css'] = '/catalog/view/theme/default/stylesheet/extension/payment/vtbpay.css';
        $data['js'] = '/catalog/view/theme/default/js/extension/payment/vtbpay.js';
        $data['desc'] = $this->config->get('payment_vtbpay_desc') ?: '';
        return $this->load->view('extension/payment/vtbpay', $data);
    }


    /**
     * Confirm function
     *
     * Initiates a payment transaction with VTB Pay and redirects the user to the VTB Pay payment page.
     * Logs all VTB Pay transactions.
     *
     * @return void
     */
    public function confirm()
    {
        $order_id = $this->session->data['order_id'] ?? null;
        $this->vtbpay_lib->logger->setOption('additionalCommonText', 'payment-' . rand(1111, 9999));

        try {
            if (!$order_id) {
                throw new \Exception($this->language->get('error_no_order_id'));
            }

            $this->setOrder($order_id);

            if (empty($pay_url = $this->getPayUrl())) {
                throw new \Exception($this->language->get('error_pay_url_not_available'));
            }

            $this->response->redirect($pay_url);

        } catch (\Exception | VtbPayException $e) {
            // Handle exception and log error
            $this->executeErrorScenario(
                $e,
                $order_id,
                __FUNCTION__
            );
        }
    }


    /**
     * Получает URL-адрес платежа для перенаправления.
     *
     * @return string URL-адрес платежа.
     */
    private function getPayUrl(): string
    {
        $order_id = $this->order['order_id'];
        $email = $this->order['email'] ?: $this->config->get('payment_vtbpay_email_fiscal');
        $total = $this->order['total'];
        $return_url = $this->url->link('extension/payment/vtbpay/return'); // Get return URL
        $two_stage = (bool) $this->config->get('payment_vtbpay_two_stage');
        $enable_fiscal = (bool) $this->config->get('payment_vtbpay_enable_fiscal');

        $items = [];
        if (!$two_stage && $enable_fiscal) $items = $this->getItems();

        $this->model_checkout_order->addOrderHistory(
            $order_id,
            1,
            'orderId: ' . $order_id,
            true
        );

        // Logging input
        $this->vtbpay_lib->logger->debug(
            __FUNCTION__ . ' > getOrderLink - INPUT: ', [
            'arguments' => [
                'order_id' => $order_id,
                'email' => $email,
                'time' => time(),
                'total' => $total,
                'return_url' => $return_url,
                'two_stage' => $two_stage,
                'items' => $items
            ]
        ]);

        $response = $this->vtbpay_lib->api->getOrderLink(
            $order_id,
            $email,
            time(),
            $total,
            $return_url,
            $two_stage,
            $items
        );

        // Logging output
        $this->vtbpay_lib->logger->debug(
            __FUNCTION__ . ' > getOrderLink - OUTPUT: ', [
            'response' => $response
        ]);

        return $response['object']['payUrl'] ?? '';
    }


    /**
     * Получает массив товаров заказа и доставку.
     *
     * @return array Массив товаров.
     */
    private function getItems(): array
    {
        $this->load->model('catalog/product');

        // Настройки по умолчанию для фискальных параметров
        $fiscal_settings = [
            'payment_vtbpay_measure_fiscal' => 0,
            'payment_vtbpay_tax_type_fiscal' => 'none',
            'payment_vtbpay_payment_type_fiscal' => 'full_prepayment',
            'payment_vtbpay_payment_subject_fiscal' => 1
        ];
        $items = [];
        $key = 1;
        // Добавление в массив товаров
        foreach ($this->order['products'] as $item) {
            $product = $this->model_catalog_product->getProduct($item['product_id']);

            $product_fiscal = [];
            // Получение фискальных параметров для каждого продукта
            foreach ($fiscal_settings as $name => $default) {
                $product_fiscal[$name] = $product[$name] ?: '-';
                if ($product_fiscal[$name] === '-') $product_fiscal[$name] = $this->config->get($name) ?: $default;
            }

            $items[] = [
                'positionId' => $key,
                'name' => $item['name'],
                'code' => (string) $item['product_id'],
                'price' => floatval($item['price']),
                'measure' => (int) $product_fiscal['payment_vtbpay_measure_fiscal'],
                'quantity' => (int) $item['quantity'],
                'taxParams' => [
                    'taxType' => $product_fiscal['payment_vtbpay_tax_type_fiscal']
                ],
                'paymentType' => $product_fiscal['payment_vtbpay_payment_type_fiscal'],
                'paymentSubject' => (int) $product_fiscal['payment_vtbpay_payment_subject_fiscal'],
                'amount' => floatval($item['total'])
            ];

            $key++;
        }

        // Добавление в массив доставки
        foreach ($this->order['totals'] as $total) {
            if ($total['code'] == 'shipping' && $total['value'] > 0) {
                $items[] = [
                    'positionId' => $key,
                    'name' => $total['title'],
                    'price' => floatval($total['value']),
                    'measure' => 0,
                    'quantity' => 1,
                    'taxParams' => [
                        'taxType' => $this->config->get('payment_vtbpay_tax_type_fiscal') ?: 'none'
                    ],
                    'paymentType' => $this->config->get('payment_vtbpay_payment_type_delivery_fiscal') ?: 'full_prepayment',
                    'paymentSubject' => 4,
                    'amount' => floatval($total['value'])
                ];

                $key++;
            }
        }

        return $items;
    }


    /**
     * Return function
     *
     * Processes the callback from VTB Pay, updates the order status based on the callback data,
     * and prepares and displays a success or failure message to the user.
     *
     * @return void
     */
    public function return()
    {
        $order_id = $this->request->get['orderId'] ?? null;
        $this->vtbpay_lib->logger->setOption('additionalCommonText', 'return-' . rand(1111, 9999));

        // Logging $_REQUEST
        $this->vtbpay_lib->logger->debug(
            'redirect > ' . __FUNCTION__ . ': ', [
            'request_data' => $_REQUEST
        ]);

        try {
            if (empty($order_id)) {
                throw new \Exception($this->language->get('error_no_order_id'));
            }

            $this->setOrder($order_id);

            $this->changeStatus();

            $this->cart->clear();
            $this->response->redirect($this->url->link('checkout/success'));

        } catch (\Exception | VtbPayException $e) {
            // Handle exception and log error
            $this->executeErrorScenario(
                $e,
                $order_id,
                __FUNCTION__
            );
        }
    }


    /**
     * Обработчик callback, вызываемый платёжкой при изменение статуса платежа.
     *
     * @return void
     */
    public function webhook(): void
    {
        $php_input = json_decode(file_get_contents('php://input'), true) ?: null;
        $this->vtbpay_lib->logger->setOption('additionalCommonText', 'webhook-' . rand(1111, 9999));

        // Logging php input
        $this->vtbpay_lib->logger->debug(
            'callback > ' . __FUNCTION__ . ': ', [
            'php_input' => $php_input
        ]);

        $order_id = $php_input['object']['orderId'] ?? '';

        try {
            if (empty($order_id)) {
                throw new \Exception($this->language->get('error_no_order_id'));
            }

            $this->setOrder($order_id);

            $this->changeStatus();

            $this->response->setOutput('OK');

        } catch (\Exception | VtbPayException $e) {
            // Handle exception and log error
            $this->executeErrorScenario(
                $e,
                $order_id,
                __FUNCTION__,
                false
            );
        }
    }


    /**
     * Получени информации о платеже и зменение статуса заказа
     *
     * @return void
     */
    public function changeStatus(): void
    {
        $payment_status = $this->getPaymentStatus();
        $current_order_status = $this->order['order_status_id'];
        $order_id = $this->order['order_id'];
        $failed_order_status = $this->config->get('payment_vtbpay_failed_transaction_status');
        $availableStatuses = [
            'PAID' => $this->config->get('payment_vtbpay_successful_transaction_status'),
            'PENDING' => '1',
            'REFUNDED' => '13'
        ];
        $order_status = $availableStatuses[$payment_status] ?? '';

        if (!empty($order_status)) {
            if ($current_order_status !== $order_status) {
                $this->model_checkout_order->addOrderHistory(
                    $order_id,
                    $order_status,
                    $payment_status,
                    true
                );
            }
        }
        else {
            if ($current_order_status !== $failed_order_status) {
                $this->model_checkout_order->addOrderHistory(
                    $order_id,
                    $failed_order_status,
                    $payment_status
                );
            }
            throw new \Exception($this->language->get('error_payment_not_paid'));
        }
    }


    /**
     * Получение статуса заказа из VTB API.
     *
     * @return string Значение статуса заказа, если оно существует, или пустая строка в противном случае.
     */
    private function getPaymentStatus(): string
    {
        $order_id = $this->order['order_id'];

        // Logging input
        $this->vtbpay_lib->logger->debug(
            __FUNCTION__ . ' > getOrderInfo - INPUT: ', [
            'arguments' => [
                'order_id' => $order_id
            ]
        ]);
        $response = $this->vtbpay_lib->api->getOrderInfo($order_id);

        // Logging output
        $this->vtbpay_lib->logger->debug(
            __FUNCTION__ . ' > getOrderInfo - OUTPUT: ', [
            'response' => $response
        ]);

        return $response['object']['status']['value'] ?? '';
    }


    /**
     * Инициализируем свойство order результатом выполнения функции getOrder.
     *
     * @param int $order_id Идентификатор заказа.
     *
     * @return self|Exception Объект заказа, если он существует, или Exception в противном случае.
     */
    private function setOrder(int $order_id): self
    {
        $this->load->model('checkout/order');

        $this->order = $this->model_checkout_order->getOrder($order_id);
        if (!$this->order) {
            throw new \Exception($this->language->get('error_order_not_found'));
        }

        $this->order['products'] = $this->model_checkout_order->getOrderProducts($order_id);
        $this->order['totals'] = $this->model_checkout_order->getOrderTotals($order_id);

        return $this;
    }


    /**
     * Сценарий, выполняемый после отлова исключения
     *
     * @param \Exception|VtbPayException $e   Объект исключения.
     * @param int $order_id                   Идентификатор заказа.
     * @param string $caller_func             Название функции, котоорая вызывает обработчик исключений.
     * @param bool $make_redirect             Совершать редирект на страницу ошибки или нет.
     *
     * @return void
     */
    private function executeErrorScenario(
        $e,
        $order_id,
        $caller_func = '',
        $make_redirect = true
    ): void {
        $context = [];
        if (method_exists($e, 'getContext')) $context = $e->getContext();

        $this->vtbpay_lib->logger->error(sprintf(
            $caller_func . ' > ' . __FUNCTION__ . ' > VtbPay exception : %s; Order id: %s;',
            $e->getMessage(),
            $order_id ?: ''
        ), $context);

        if ($make_redirect) {
            $this->session->data['error'] = $e->getMessage();
            $this->response->redirect($this->url->link('checkout/failure', '', true));
        }
        else {
            $this->response->setOutput($e->getMessage());
        }
    }
}
